==> 1.0.0.tar.gz: lib/src/sub_types/NonLabelledStatements.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';

import '../core/Identifier.dart';
import '../main/Statement.dart';
import '../main/Expression.dart';
import '../main/InitializedVariableDeclaration.dart';

/*
See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
Page 145-165
*/

/* <localVariableDeclaration> ::= <initializedVariableDeclaration> ';' */
class LocalVariableDeclaration extends ParserType {
  InitializedVariableDeclaration initializedVariableDeclaration;

  LocalVariableDeclaration(Lexer lexer) {
    initializedVariableDeclaration = InitializedVariableDeclaration(lexer);
    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return '$initializedVariableDeclaration;';
  }

}

/* <whileStatement> ::= while '(' <expression> ')' <statement> */
class WhileStatement extends ParserType {
  Expression expression;
  Statement statement;

  WhileStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'while');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    expression = Expression(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();

    statement = Statement(lexer);
  }

  String toString() {
    return 'while ($expression) $statement';
  }

}

/* <doStatement> ::= do <statement> while '(' <expression> ')' ';' */
class DoStatement extends ParserType {
  Expression expression;
  Statement statement;

  DoStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'do');
    lexer.advance();

    statement = Statement(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'while');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    expression = Expression(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'do $statement while ($expression);';
  }

}

/* <ifStatement> ::= if '(' <expression> ')' <statement> (else <statement>)? */
class IfStatement extends ParserType {
  Expression expression;
  Statement statement1;
  Statement statement2;

  IfStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'if');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    expression = Expression(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();

    statement1 = Statement(lexer);

    int snapShot = lexer.currentIndex;
    if (isType(lexer, TType.KEYWORD) && isVal(lexer, 'else')) {
      lexer.advance();
      try {
        statement2 = Statement(lexer);
      }
      on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }
  }

  String toString() {
    return 'if ($expression) $statement1' + (statement2 != null ? ' $statement2' : '');
  }

}

/* <rethrowStatement> ::= rethrow ';' */
class RethrowStatement extends ParserType {

  RethrowStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'rethrow');
    lexer.advance();
    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'rethrow;';
  }

}

/* <breakStatement> ::= break <identifier>? ';' */
class BreakStatement extends ParserType {
  Identifier identifier;

  BreakStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'break');
    lexer.advance();

    int snapShot = lexer.currentIndex;
    try {
      identifier = Identifier(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'break' + (identifier != null ? ' $identifier;' : ';');
  }

}

/* <continueStatement> ::= continue <identifier>? ';' */
class ContinueStatement extends ParserType {
  Identifier identifier;

  ContinueStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'continue');
    lexer.advance();

    int snapShot = lexer.currentIndex;
    try {
      identifier = Identifier(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'continue' + (identifier != null ? ' $identifier;' : ';');
  }

}

/* <returnStatement> ::= return <expression>? ';' */
class ReturnStatement extends ParserType {
  Expression expression;

  ReturnStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'return');
    lexer.advance();

    int snapShot = lexer.currentIndex;
    try {
      // print ('Started Expression');
      expression = Expression(lexer);
      // print ('Finished Expression');
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'return' + (expression != null ? ' $expression;' : ';');
  }

}

/* <yieldStatement> ::= yield <expression> ';' */
class YieldStatement extends ParserType {
  Expression expression;

  YieldStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'yield');
    lexer.advance();

    int snapShot = lexer.currentIndex;
    try {
      expression = Expression(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'yield' + (expression != null ? ' $expression;' : ';');
  }

}

/* <yieldEachStatement> ::= yield '*' <expression> ';' */
class YieldEachStatement extends ParserType {
  Expression expression;

  YieldEachStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'yield');
    lexer.advance();
    checkType(lexer, TType.MULTIPLY);
    lexer.advance();

    int snapShot = lexer.currentIndex;
    try {
      expression = Expression(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'yield*' + (expression != null ? ' $expression;' : ';');
  }

}

/* <expressionStatement> ::= <expression>? ';' */
class ExpressionStatment extends ParserType {
  Expression expression;

  ExpressionStatment(Lexer lexer) {
    int snapShot = lexer.currentIndex;
    try {
      expression = Expression(lexer);
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return expression != null ? '$expression;' : ';';
  }

}

==> 1.1.1.tar.gz: lib/src/sub_types/NonLabelledStatements.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';

import '../core/Identifier.dart';
import '../main/Statement.dart';
import '../main/Expression.dart';
import '../main/InitializedVariableDeclaration.dart';

/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 145-165

/// <localVariableDeclaration> ::= <initializedVariableDeclaration> ';'
class LocalVariableDeclaration extends ParserType {
  InitializedVariableDeclaration initializedVariableDeclaration;

  LocalVariableDeclaration(Lexer lexer) {
    initializedVariableDeclaration = InitializedVariableDeclaration(lexer);
    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return '$initializedVariableDeclaration;';
  }
}

/// <whileStatement> ::= while '(' <expression> ')' <statement>
class WhileStatement extends ParserType {
  Expression expression;
  Statement statement;

  WhileStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'while');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    expression = Expression(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();

    statement = Statement(lexer);
  }

  String toString() {
    return 'while ($expression) $statement';
  }
}

/// <doStatement> ::= do <statement> while '(' <expression> ')' ';'
class DoStatement extends ParserType {
  Expression expression;
  Statement statement;

  DoStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'do');
    lexer.advance();

    statement = Statement(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'while');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    expression = Expression(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'do $statement while ($expression);';
  }
}

/// <ifStatement> ::= if '(' <expression> ')' <statement> (else <statement>)?
class IfStatement extends ParserType {
  Expression expression;
  Statement statement1;
  Statement statement2;

  IfStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'if');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    expression = Expression(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();

    statement1 = Statement(lexer);

    int snapShot = lexer.currentIndex;
    if (isType(lexer, TType.KEYWORD) && isVal(lexer, 'else')) {
      lexer.advance();
      try {
        statement2 = Statement(lexer);
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }
  }

  String toString() {
    return 'if ($expression) $statement1' +
        (statement2 != null ? ' $statement2' : '');
  }
}

/// <rethrowStatement> ::= rethrow ';'
class RethrowStatement extends ParserType {
  RethrowStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'rethrow');
    lexer.advance();
    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'rethrow;';
  }
}

/// <breakStatement> ::= break <identifier>? ';'
class BreakStatement extends ParserType {
  Identifier identifier;

  BreakStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'break');
    lexer.advance();

    int snapShot = lexer.currentIndex;
    try {
      identifier = Identifier(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'break' + (identifier != null ? ' $identifier;' : ';');
  }
}

/// <continueStatement> ::= continue <identifier>? ';'
class ContinueStatement extends ParserType {
  Identifier identifier;

  ContinueStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'continue');
    lexer.advance();

    int snapShot = lexer.currentIndex;
    try {
      identifier = Identifier(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'continue' + (identifier != null ? ' $identifier;' : ';');
  }
}

/// <returnStatement> ::= return <expression>? ';'
class ReturnStatement extends ParserType {
  Expression expression;

  ReturnStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'return');
    lexer.advance();

    int snapShot = lexer.currentIndex;
    try {
      // print ('Started Expression');
      expression = Expression(lexer);
      // print ('Finished Expression');
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'return' + (expression != null ? ' $expression;' : ';');
  }
}

/// <yieldStatement> ::= yield <expression> ';'
class YieldStatement extends ParserType {
  Expression expression;

  YieldStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'yield');
    lexer.advance();

    int snapShot = lexer.currentIndex;
    try {
      expression = Expression(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'yield' + (expression != null ? ' $expression;' : ';');
  }
}

/// <yieldEachStatement> ::= yield '*' <expression> ';'
class YieldEachStatement extends ParserType {
  Expression expression;

  YieldEachStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'yield');
    lexer.advance();
    checkType(lexer, TType.MULTIPLY);
    lexer.advance();

    int snapShot = lexer.currentIndex;
    try {
      expression = Expression(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return 'yield*' + (expression != null ? ' $expression;' : ';');
  }
}

/// <expressionStatement> ::= <expression>? ';'
class ExpressionStatment extends ParserType {
  Expression expression;

  ExpressionStatment(Lexer lexer) {
    int snapShot = lexer.currentIndex;
    try {
      expression = Expression(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return expression != null ? '$expression;' : ';';
  }
}
